/*
 * $Log: CCB.java,v $
 * Revision 1.4  2002/01/29 21:41:07  stuart
 * setOwner()
 *
 * Revision 1.3  2000/06/27 16:25:36  stuart
 * pccbdw, getLogin
 *
 * Revision 1.2  1999/08/11  23:35:26  stuart
 * more methods
 *
 */
package bmsi.edx;

/** A class to represent a CCB in EDX memory.
    @author Stuart D. Gathman
 */

public class CCB implements Edxccb {
  protected final EDX edx;
  protected final char ccba;

  /** This class can only be instantiated from this package.
   */

  CCB(EDX edx,char ccb) {
    this.edx = edx;
    ccba = ccb;
  }

  /** Return the name of this EDX terminal.  The name is padded to 8 chars,
  use <code>String.trim()</code> as required.
   */
  public String getName() {
    return edx.etoa(0,ccba + CCBNAME,8);
  }

  /** Return the user id of the EDX4978 interface program for this
     EDX terminal. */
  public final int getUser() { return gccba(CCBUID); }

  /** Return the group id of the EDX4978 interface program for this
     EDX terminal. */
  public final int getGroup() { return gccba(CCBGID); }

  final int gccba(int off) { return edx.getAddr(0,ccba + off); }
  final int gccbw(int off) { return edx.getWord(0,ccba + off); }
  final void pccbw(int off,int val) { edx.putWord(0,ccba + off,val); }
  final void pccbb(int off,int val) { edx.putByte(0,ccba + off,val); }
  final void pccbdw(int off,int val) { edx.putDword(0,ccba + off,val); }

  /** Return the process id of the EdX terminal client. */
  public final int getPID() { return edx.getDword(0,ccba + CCBPID); }

  /** Return whether this CCB is owned by an ActiveTCB. */
  public final boolean ownedBy(ActiveTCB tcb) {
    return gccba(CCBQUIN) == tcb.acttcb && gccbw(CCBQUIN + 2) == tcb.actkey;
  }

  /** Return the EDX security record for this CCB.
    @since 1.3
    @return null if EDX security is not active
   */
  public final EDXLogin getLogin() { return EDXLogin.getLogin(edx,ccba); }

  private CCBOwner owner;

  public synchronized void setOwner(CCBOwner o) {
    if (owner != null)
      owner.deactivated(this);
    owner = o;
    if (o != null)
      o.activated(this);
  }
}
