/*
 * $Log: EDXLPrint.java,v $
 * Revision 1.2  2002/01/29 21:40:29  stuart
 * More Docs
 *
 */
package bmsi.edx;

import bmsi.edx.fs.FSPager;
import bmsi.xopen.FType;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.IOException;

/** Implement CALL LPRINT in Java.  Reimplementing this EDL library 
  function in Java lets us do fancy things with spooling and previewing
  by simply relinking the EDL report.  The associated EDL stub source
  is in lprt.e in this source directory.
  <p>
  We use FSPager to support left/right scrolling and eventually
  scroll back.  
  <p>
  A proposed CALL LCHECK,N will let EDL programs start a
  new page if fewer than N lines remain on the current page.  
  Printing a header from EDL will force a new page and install a new
  header template.  The template will have variables which will be substituted
  with pageno and other things (company name, date, etc.).
  <p>
  By keeping the EDL program in the dark about pagesize, reports
  can be printed from the scroll back buffer without rerunning the EDL
  code that generated them.
  EDL programs which do not use the new pagination mechanism will still work,
  but will not be able to be repaginated.
  <p>
  BUGS: the EDX4978 object needs to live across EdxjavaRequest instances
  	for this application, so we don't dispose any EDX4978 in
	EdxjavaRequest.dispose().  Somehow, GLCHPT with the new LPRINT
	causes <code>bmsi.edx.EDXError: JWAITKEY with old version of edxjava.edx
	</code> at a random point.
	<p>
	This is a work in progress.

  @author Stuart D. Gathman
  Copyright (C) 2000 Business Management Systems, Inc.
 */

class EDXLPrint implements EDXServlet {
  private final static String compile = config.BMSTimeStamp.date;
  private final static String version
    = "$Id: EDXLPrint.java,v 1.2 2002/01/29 21:40:29 stuart Exp $";
  private int txta;
  private OutputStream out;
  private PrintWriter prt;
  private int pagesize;
  private int pline;
  private static final SelectPrinter selprtr = new SelectPrinter();

  public void init() { }
  public void destroy() { }

  private void skip(int cnt) {
    while (cnt-- > 0) {
      prt.println();
      ++pline;
    }
  }

  private void tof(int fc) {
    if (fc <= pline) {
      pline = (pline - 1) % pagesize;
      if (pline > 0)
        skip(pagesize - pline);
      pline = 1;
    }
    skip(fc - pline);
  }

  /** Remove trailing blanks.  String.trim() removes leading blanks
      as well.  */
  private static String dtb(String s) {
    int len = s.length();
    while (len > 0 && Character.isWhitespace(s.charAt(len-1)))
      --len;
    return len == s.length() ? s : s.substring(0,len);
  }

  private int lprint(String txt,int fc) {
    if (fc == 0) {
      System.err.println("CALL LPRINT,0");
      prt.close();
      return 0;
    }
    if (fc > 0) tof(fc);
    prt.print(dtb(txt));
    if (fc < 0)
      skip(-fc);
    return prt.checkError() ? 0 : pline;
  }

  private int qprint(int txt,String pname,EDXServletRequest req)
    throws IOException {
    txta = txt;
    out = selprtr.getOutputStream(pname);
    if (out == null) {
      out = new FSPager(req.getScreen());
      pagesize = 23;
    }
    else
      pagesize = 66;
    prt = new PrintWriter(new OutputStreamWriter(out));
    System.err.println("pagesize = " + pagesize + ", pname = " + pname);
    return pagesize;
  }

  public short callJava(EDXServletRequest req) throws EDXServlet.Errpost {
    byte[] b = req.getParm();
    try {
      if (b.length == 2) {
	int fc = FType.ldshort(b,0);
	ActiveTCB tcb = req.getTCB();
	String txt = null;
	if (fc != 0) {
	  txt = tcb.getText(txta);
	  tcb.atoe(txta,tcb.getByte(txta - 2),"");	// clear app txt buf
	}
	return (short)lprint(txt,fc);
      }
      if (b.length == 6) {
	int txt = FType.ldushort(b,0);
	int iocb = FType.ldushort(b,2);
	String prtr = req.getTCB().etoa(iocb,8).trim();
	return (short)qprint(txt,prtr,req);
      }
    }
    catch (IOException x) {
      System.err.println(x.toString());
      throw new EDXServlet.Errpost(0,"Error opening printer");
    }
    throw new EDXServlet.Errpost(0,"Invalid EDXLPrint call from EDX");
  }
}
