/* $Log: EDXLogin.java,v $
/* Revision 1.1  2002/01/30 19:25:58  stuart
/* Convert to CVS.
/*
 */
package bmsi.edx;

/** The security data for an EDX CCB.
    @see CCB@getLogin
    @author Stuart D. Gathman
    Copyright (C) 1999 Business Management Systems, Inc.
 */
public class EDXLogin {
  private ActiveTCB edx;
  /** The CCB address for this security record. */
  public final char ccb;
  /** The current user id on this CCB. */
  public short id;
  /** The privilege bit mask for the current user on this CCB. */
  public int userpriv;
  /** The port number assigned by the login program.  Port numbers 
     remain stable even when the EDX VM is recompiled (thereby changing
     CCB addresses).
   */
  public short port;
  /** The privilege bit mask for this CCB. */
  public int termpriv;
  /** The group of the current user on this CCB. */
  public short group;

  private EDXLogin(ActiveTCB edx,char ccb) {
    this.edx = edx;
    this.ccb = ccb;
    msread();
  }

  static EDXLogin getLogin(EDX edx,char ccb) {
    PROG wtr = edx.wheres("#WAITER");
    if (wtr == null) return null;
    return new EDXLogin(wtr.edx,ccb);
  }

  // $PARM1 - 0,1,-1 IPL/TIME SET SWITCH
  private static final short WTPARM = 0x0076;
  private static final short WTQCB  = WTPARM+2; // FILE QUEUE
  private static final short WTBUF  = WTQCB+10; // TERMINAL STATUS BUFFERS
  private static final short WTPTR  = WTPARM-2;

  private int find() {
    PROG wtr = edx.edx.wheres("#WAITER");
    if (wtr == null) return 0;
    edx = wtr.edx;
    int stg = edx.getAddr(wtr.plp + Edxprg.PRGSTG);
    int los = edx.getWord(wtr.plp + Edxprg.PRGLOS);
    if (los == 0) {
      los = 1;
      stg = wtr.plp + WTBUF;
    }
    int cnt = los * 256 / MSLEN;
    synchronized (edx.edx) {
      for (int i = 0,adr = stg; i < cnt; ++i,adr += MSLEN) {
	if (ccb == edx.getAddr(adr))
	  return adr;
      }
      for (int i = 0,adr = stg; i < cnt; ++i,adr += MSLEN) {
	if (edx.getAddr(adr) == 0) {
	  edx.putWord(adr,ccb);
	  return adr;
	}
      }
    }
    return 0;
  }

  /** Refresh the security data for this CCB from the EDX VM.  */
  public void msread() {
    int tsadr = find();
    if (tsadr != 0) {
      id = edx.getWord(tsadr + MSID);
      userpriv = edx.getDword(tsadr + MSPRIV);
      port = edx.getWord(tsadr + MSPORT);
      termpriv = edx.getDword(tsadr + MSTYPE);
      group = edx.getWord(tsadr + MSACCT);
    }
    else {
      id = 0;
      userpriv = 0;
      port = 0;
      termpriv = 0;
      group = 0;
    }
  }

  /** Update the security data for this CCB in the EDX VM. */
  public void mswrite() {
    int tsadr = find();
    if (tsadr == 0)
      throw new EDXError("EDX Security table overflow");
    edx.putWord(tsadr + MSID,id);
    edx.putDword(tsadr + MSID,userpriv);
    edx.putWord(tsadr + MSPORT,port);
    edx.putDword(tsadr + MSTYPE,termpriv);
    edx.putWord(tsadr + MSACCT,group);
  }

  //		    *** TERMINAL STATUS EQUATES
  /** (2) C VALIDATION TAG = CCB ADDRESS */
  private final static short MSTAG = 0;
  /** (2) W USER ID NO. 0 = */
  private final static short MSID = 2;
  /** (4) D PRIVILEDGES OF THIS USER */
  private final static short MSPRIV = 4;
  /** (2) W TERMINAL PORT NUMBER */
  private final static short MSPORT = 8;
  /** (4) D PRIVILEDGES OF THIS PARTICULAR TERM */
  private final static short MSTYPE = 10;
  /** (2) W USER GROUP ACCOUNT NUMBER */
  private final static short MSACCT = 14;
  private final static short MSLEN = 16;
  // Privileges used 
  //   Domestic - Ok to go from o/e to r/t			x'00000001'
  // Privileges used at JSC 
  //   apedit: 1 - Edit File number, artran#		x'00000008'
  //   apedit: 2 - Edit voucher data after paid		x'00000004'
  //   unpay 						x'00000002'
  //   accounting functions				x'00000001'
  //   arcuede: Restrict Customers				x'80000000'
  //   apvnede: Restrict Vendors				x'40000000'
  //   arcuede: Restrict who can enter/chg slsrep		x'20000000'
  //   abi - Statement deletions				x'00800000'
  //   manager override for credit memos			x'00400000'
  //   manager override for customer complaints		x'00200000'
  //	 USER BITS	 (MSPRIV)
  /** SYSTEM CAPABILITY */
  public final static int MSSYSTEM = 0x80000000;
  /** SET TIME */
  public final static int MSTIME = 0x40000000;
  /** MULTIPLE LOGON */
  public final static int MSMLOG = 0x20000000;
  /** SYSTEM MANAGER */
  public final static int MSMNGR = 0x10000000;
  /** ENTER NEW STUDENTS */
  public final static int MSNSTU = 0x00800000;
  /** BISYNC */
  public final static int MSBSC = 0x00080000;
  /** WORD PROCESSING */
  public final static int MSWP = 0x00040000;
  /** TELEX PRIVELEDGE */
  public final static int MSTLX = 0x00020000;
  /** MESSAGE CENTER */
  public final static int MSMAIL = 0x00010000;
  //
  //	 ACCOUNTING PRIVILEDGES
  /** MAILING LIST */
  public final static int MSNL = 0x0800;
  /** ORDER ENTRY */
  public final static int MSOE = 0x0400;
  /** PAYROLL */
  public final static int MSPA = 0x0200;
  /** ACCOUNTS PAYABLE */
  public final static int MSAP = 0x0100;
  /** ACCOUNMS RECEIVABLE */
  public final static int MSAR = 0x0080;
  /** GENERAL LEDGER */
  public final static int MSGL = 0x0040;
  //
  //	 SUBSYTEM MENU PRIVILEDGES
  /** ACCOUNTING */
  public final static int MSMNA = 0x0020;
  /** UTILITIES */
  public final static int MSMNU = 0x0010;
  /** MENU 'P' */
  public final static int MSMNP = 0x0008;
  /** INQUIRIES */
  public final static int MSMNQ = 0x0004;
  /** SECURITY */
  public final static int MSMNS = 0x0002;
  //
  //	 TERMINAL BITS	 (MSTYPE)
  /** ALLOW LOGINS */
  public final static int MSLOGIN = 0x40000000;
  /** FULL SCREEN TERMINAL */
  public final static int MSSCRN = 0x20000000;
  /** 4978 TERMINAL */
  public final static int MS4978 = 0x10000000;
  /** UNSECURE TERMINAL */
  public final static int MSSEC = 0x08000000;
  /** MAKE MASK FOR HARD COPY PASSWORD */
  public final static int MSHDCPY = 0x04000000;
  //      MENU PRIVILEDGES   FOR KUWAIT
  /** ACCESS TO OFFICE ACCOUNT# 100001-100015 */
  public final static int MSLGR = 0x0800;
  /** ACCESS TO OFFICE ACCOUNT# 100003 ONLY */
  public final static int MSLGR3 = 0x0400;
  /** ACADEMIC MENU */
  public final static int MSAD = 0x0200;
  /** ACCOUNTING MENU */
  public final static int MSAC = 0x0100;
  // MSNSTU   EQU X'0080'	ENTER NEW STUDENTS
  /** EDIT STUDENTS */
  public final static int MSESTU = 0x0040;
  //
}
