package bmsi.edx;

/** An interface to an EDX CCB. This interface is suitable for implementing
   an EDX4974 or EDX4978 in Java.  Note that the EDX4978 class lets Java
   applications use the implementation - whether in C or Java - whereas
   this class lets Java provide the implementation.

   @author Stuart D. Gathman
   Copyright (C) 2000 Business Management Systems, Inc.
 */
public class EDXTerm extends EDX4978 {
  // FIXME: inherit from a common base class rather than from EDX4978
  private int pid = posix.IPC.pid;
  private ActiveTCB tcb;
  private boolean local;
  private boolean fldexit;

  public EDXTerm(EDX edx,char ccba) {
    super(edx,ccba);
    // FIXME: register with Edxjava
    pccbw(CCBVFLG,0x1000);	// tell edxnuc 4978 driver to use senduser
    pccbdw(CCBPID,pid);
  }

  public void dispose() {
    pccbdw(CCBPID,0);
    pccbdw(CCBVFLG,0);
  }

  void calluser(Edxuser.UIMsg m) throws EDXException {
    tcb = m.getActiveTCB();
    char ccba = m.getParm();
    if (ccba != this.ccba)
      throw new EDXException("EDXTerm: wrong CCB");
    // FIXME: route message to EDXTerm instance based on ccba
    switch (gccbw(CCBVSCD)) {
    case DEVBELL:
      beep();
      // fall through
    case DEVUPDATE:
      devpost(true);
      break;
    case DEVWAITKEY:
      local = true;
      fldexit = gccba(CCBINST) == 0xA030;
      // empty type ahead queue
//    while (keyq.count) {
//	localkey(keyq.get());
//	if (!local) return;
//    }
      setUpdate(0);
      //update(sp->first,sp->last);
      //sp->first = sp->msiz;
      //sp->last = 0;
      //sync();
      break;
    case DEVCCALL:
      char adr = (char)gccba(CCBPRM1);
      if (adr != 0)
        post(-1);
      int retcode = ccall();
      dqterm(-1);
      if (adr != 0)
        tcb.post(adr,retcode);
      else
        post(retcode);
      break;
    }
  }

  private void devpost(boolean dqflag) throws EDXException {
    int flg = getUpdate();
    int code = dqflag ? 1 : -1;
    if (dqflag)
      dqterm(-1);
    if (flg != 2) post(code);
    s1sync();
    if (flg == 2) post(code);
  }

  /** Make a sound. */
  protected void beep() { }

  private int ccall() {
    return 1;	// always fail until implemented
  }

  private void post(int code) throws EDXException {
    edx.post((short)0,(char)(ccba + CCBECB),(short)code);
  }

  private void dqterm(int code) throws EDXException {
    if (ownedBy(tcb))
      edx.deq((short)0,(char)(ccba+CCBQCB),(short)code);
  }

}
