package bmsi.edx;

/** Define the fields of an EDX Console Control Block (CCB).
 */
public interface Edxccb {
  public static final int
    /**
    * real CCB fields
    **/
    CCB3DC0 = -152,	/* 3rd 8-word DCB */
    CCBDCB0 = -90,	/* DCB word 0 */
    CCBDCB1 = -88,	/* DCB word 1 */
    CCBDCB2 = -86,	/* DCB word 2 */
    CCBDCB3 = -84,	/* DCB word 3 */
    CCBDCB4 = -82,	/* DCB word 4 */
    CCBDCB5 = -80,	/* DCB word 5 */
    CCBDCB6 = -78,	/* DCB word 6 */
    CCBDCB7 = -76,	/* DCB word 7 */
    CCBPREP = -74,	/* prepare I/O IDCB */
    CCBRSET = -70,	/* reset IDCB*/
    CCBRDID = -66,	/* read-id IDCB */
    CCBSTRT = -62,	/* start-cs IDCB */
    CCBWRIT = -62,	/* write-data IDCB */
    CCBSTST = -58,	/* start-cs-status IDCB */
    CCBREAD = -58,	/* read-data IDCB */

    CCBLDEL = -54,	/* line-delete character */
    CCBCDEL = -53,	/* char-delete character */
    CCBSKIP = -52,	/* carriage-return, line-feed chars */
    CCBSKLN = -50,	/* number of chars in newline sequence (2 + # idles) */
    CCBIDLE = -49,	/* idle character */

    CCBFEAT = -48,	/* feature indicators */
     ATTNOFF = 0x80,	/* attention off */
     PRMTOPT = 0x08,	/* prompt option (SYNC = YES) */
     OUTPAUS = 0x04,	/* output pause on */

    CCBFET2 = -47,	/* feature byte #2 indicators */
    CCB31CH = 0x80,	/* 3101c mode */
    CCB31BL = 0x40,	/* 3101b mode */
    CCBSPLA = 0x20,	/* spoolable device */
    CCBSPLD = 0x10,	/* this is a spool device */
    CCBATRC = 0x04,	/* attn byte 0 was received */
    CCBPFRC = 0x02,	/* pf1 byte 0 was received */

    CCBOLCT = -46,	/* current output line count (history) */
    CCBCODE = -44,	/* address of code xlate table (0=EBCDIC) */

    CCBIOCB = -42,	/* IOCB (modified) */
    CCBFLGS = -42,	/* flags */
     STKMODE = 0x08,	/* static screen */
     OVERFLW = 0x02,	/* overflow lines */

     CCBTWRK = -41,	/* top of working area (line nr) */
     CCBTOPM = -40,	/* top margin */
     CCBBOTM = -39,	/* bottom margin */
     CCBLFTM = -38,	/* left margin */
     CCBPSIZ = -37,	/* current page size */
     CCBBLEN = -36,	/* current line size */
     CCBNXTL = -34,	/* next line position (NOT relative to CCBTOPM) */
     CCBCPOS = -33,	/* cursor displacement (NOT relative to CCBLFTM) */
     CCBBSIZ = -32,	/* current buffer size */
     CCBIICB = -30,	/* initial IOCB */
     CCBPMAX = -18,	/* maximum page size */
     CCBLMAX = -17,	/* maximum line size */
     CCBNAME = -16,	/* terminal name (EBCDIC) */

     CCBOPTS = -8,	/* current instruction options */
     LENPARM = 0x8000,	    /* length parameter */
     CLRLINE = 0x4000,	    /* clear line at attn */
     NOXLATE = 0x2000,	    /* no ascii translation */
     PROTECT = 0x1000,	    /* write protected character / read no echo */
     TYPMASK = 0x0C00,	    /* mask for type parameter: */
     DATA = 0x0000,		/* type = DATA */
     ALL = 0x0400,		/* type = ALL */
     MODDATA = 0x0800,		/* type = MODDATA */
     MODALL = 0x0C00,		/* type = MODALL */
     WAITIND = 0x0200,	    /* wait indicator */
     LOCALB = 0x0100,	    /* local buffer */
     INPUT = 0x0080,	    /* input instruction */
     HARDCPY = 0x0040,	    /* hardcopy instruction */
     WAITKEY = 0x0020,	    /* waitkey instruction */
     DIRECT = 0x0008,	    /* direct I/O */
     ADVINP = 0x0004,	    /* read advance input, no pop */
     LINEOUT = 0x0002,	    /* mode = LINE (output) */
     LINEINP = 0x0001,	    /* mode = LINE (input) */

     CCBERCD = -6,	/* errro code */
     CCBERCT = -4,	/* error count */
     CCBLINK = -2,	/* link to next CCB (0 = end) */
     CCBIA = 0,	/* immediate action address */
     CCBDVID = 2,	/* device ID */
     CCBIOS = 4,	/* start-I/O entry */
     CCBEXRW = 6,	/* index to read/write entry (IOSTERM) */
     CCBTAG = 8,	/* block type (-1 = CCB) */
     CCBPOST = 12,	/* call to post routine */
     CCBWAIT = 36,	/* call to wait routine */
     CCBLEN = 40,	/* length of ccb structure */
     CCBATCH = 58,	/* call to attach routine */
     CCBKBT = 80,	/* address of keyboard task */
     CCBHCPY = 82,	/* address of hardcopy or virtual CCB */
     CCBECB = 84,	/* I/O event control block */
     CCBQCB = 90,	/* I/O queue control block */
     CCBQUIN = 100,	/* pointer to TCB of user in control */
     CCBATTN = 106,	/* attention key code */
     CCBPF1 = 108,	/* base for PF keys */
     CCBFLD = 110,	/* temporary storage */
     CCBRS0 = 122,	/* reg save area 20 bytes (see above) */
     CCBS0 = 142,	/* reg save area 16 bytes (see above) */
     CCBMSIZ = 158,	/* number of chars in buffer (negative if input) */
     CCBBORG = 160,	/* current buffer start */
     CCBBFAD = 162,	/* initial buffer start */
     CCBBKEY = 164,	/* buffer address key */
     CCBSTA = 166,	/* buffer state indicators */
     CCBFLG2 = 174,	/* flag word */
     CCBPACE = 0x0008,	    /* terminal defined for XON/XOFF */
     CCBCTLB = 192,	/* two byte control buffer */
     CCBTBUF = 194;	/* system buffer */
  public final static int
    CCBPID = CCBPREP,		/* process ID of terminal emulator or 0 */
    CCBUID = CCBATCH + 0,	/* user ID of terminal emulator or 0 */
    CCBGID = CCBATCH + 2,	/* group ID of terminal emulator or 0 */
    CCBID = CCBRSET + 1,	/* CCB ID address */
    CCBTTY = CCBS0,		/* unix device name (without "/dev/") */
    CCBCPSV = (CCBRS0 + 2),	/* NXTL and CPOS save area */
    CCBRETC = (CCBRS0 + 4),	/* CCB return code for task word */
    CCBVTID = (CCBRS0 + 6),	/* virtual terminal semaphore number */
    CCBVFLG = (CCBRS0 + 8),	/* virtual terminal flag word (see devvirt.c) */
    CCBVSCD = (CCBRS0 + 10),	/* virtual terminal send code */
    CCBINST = (CCBWAIT + 0),	/* terminal emulator instruction */
    CCBPRM0 = (CCBWAIT + 2),	/* terminal emulator instruction parameter 0 */
    CCBPRM1 = (CCBWAIT + 4),	/* terminal emulator instruction parameter 1 */
    CCBPRM2 = (CCBWAIT + 6),	/* terminal emulator instruction parameter 2 */
    CCBPRM3 = (CCBWAIT + 8),	/* terminal emulator instruction parameter 3 */
    CCBPRM4 = (CCBWAIT + 10),	/* terminal emulator instruction parameter 4 */
    CCBPRM5 = (CCBWAIT + 12);	/* terminal emulator instruction parameter 5 */
}
