/*
 * $Log: EdxjavaRequest.java,v $
 * Revision 1.10  2002/03/05 04:27:07  stuart
 * Optimize runedl
 *
 * Revision 1.9  2002/03/05 03:40:57  stuart
 * Generalized runedl.
 *
 * Revision 1.8  2002/01/29 21:44:55  stuart
 * Log errpost exceptions
 *
 * Revision 1.7  2001/04/11 19:27:27  stuart
 * Provide access to Edxjava instance
 * Support ushell
 *
 * Revision 1.6  2000/09/01 21:17:40  stuart
 * Disable debugging output for terminal socket
 *
 * Revision 1.5  2000/09/01 20:58:49  stuart
 * Fix returning from terminal socket
 *
 * Revision 1.4  2000/09/01 19:00:36  stuart
 * Don't dispose EDX4978, can be retained and used by background Java code
 *
 * Revision 1.3  2000/05/18 21:23:02  stuart
 * try to reduce restrictions on TermSocket
 *
 */
package bmsi.edx;

import java.net.Socket;
import java.net.ServerSocket;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.StringTokenizer;

/** Implements EDXServletRequest for the EDX/Java interface.  Each
   Request gets its own Thread so that multiple calls can be active
   at once.
   @author Stuart D. Gathman
   Copyright (C) 1999 Business Management Systems, Inc.
 */
class EdxjavaRequest extends Thread implements EDXServletRequest {
  private byte[] b;
  private ActiveTCB edx;
  private int outlen = 0;
  private Socket sock = null;
  private EDXServlet s;
  private EdxjavaRequest next;
  private static EdxjavaRequest pool = null;
  private static Hashtable suspended = new Hashtable();
  private char addr;
  private Edxuser.UIMsg m;
  private Edxjava j;
  private EDX4978 screen = null;

  /** Return the Edxjava instance this request belongs to. */
  public Edxjava getEdxjava() { return j; }

  /** Execute servlets until interrupted. */
  public void run() {
    try {
      for (;;) {
        synchronized (this) {
	  while (s == null)
	    wait();
	}
	short rc = 0;
	short res = Edxjava.JOK;
	try {
	  rc = s.callJava(this);
	}
	catch (EDXServlet.Errpost errpost) {
	  errpost.printStackTrace();
	  res = errpost.code;
	}
	catch (EDXError err) {
	  err.printStackTrace();
	  res = Edxjava.JEDXERR;	// app got EDXError
	}
	catch (Throwable e) {
	  e.printStackTrace();
	  res = Edxjava.JAPPERR;	// uncaught app err
	}
	edx.ptcbw(Edxtcb.TCBCO2,rc);
	if (outlen > 0)
	  edx.copyIn(addr,b,0,outlen);
	dispose(res);
      }
    }
    catch (InterruptedException x) { }
  }

  /** Create a new thread for the request pool. */
  private EdxjavaRequest(ActiveTCB edx,byte[] b) {
    super("EdxjavaRequest");
    this.edx = edx;
    this.b = b;
    /* EDXServlets are notified via destroy() from the main Thread
       when exiting.
       Synchronizing both callJava() and destroy() will ensure
       that a call in progress finishes.
     */
    setDaemon(true);
    start();
  }

  /** Return a request Thread to the pool. */
  private static synchronized void putRequest(EdxjavaRequest r) {
    r.next = pool;
    pool = r;
  }

  /** Return a servlet request Thread from a pool of servlet request
      Threads.  Create a new Thread if necessary.
      @param m	The CALLUSER request.
      @param parm The SUBROUT header of the EDL JAVA stub.
      @return An EdxjavaRequest ready to run.
   */
  static synchronized EdxjavaRequest getRequest(Edxuser.UIMsg m,char parm) {
    ActiveTCB edx = m.getActiveTCB();
    char addr = edx.getAddr(parm+2);
    char len = edx.getAddr(parm+4);
    EdxjavaRequest r = pool;
    byte[] b = edx.getBytes(addr,len);
    if (r != null) {
      pool = r.next;
      r.edx = edx;
      r.b = b;
      r.outlen = 0;
    }
    else
      r = new EdxjavaRequest(edx,b);
    r.m = m;
    r.addr = addr;
    return r;
  }

  /** Run a servlet request on this Thread. */
  synchronized void run(EDXServlet s,Edxjava j) {
    if (this.s == null) {
      this.j = j;
      this.s = s;
      notify();
    }
  }

  /** Transfer control back to EDL. EDL will hopefully CALLUSER JRESUME
      eventually, at which point we return.  Suspended threads are
      identified by their ActiveTCB.  The EDL opcode is copied to EDX
      memory and executed.
      @param cmd	the single EDL opcode to execute.
   */
  void termctrl(int cmd) {
    termctrl(cmd,0);
  }

  synchronized void termctrl(int cmd1,int cmd2) {
    runedl(new char[] { (char)cmd1, (char)cmd2 });
  }

  private static final byte[] calluser = {
    (byte)0x03,(byte)0x01,
    (byte)0xC3,(byte)0xD4,(byte)0xE6,(byte)0xC1,(byte)0xC1,(byte)0x40,
    (byte)0x80, (byte)0x01
  };

  synchronized void runedl(char[] cmds) {
    closeTermSocket();
    final char edlpc = edx.gtcba(Edxtcb.TCBS1);
    if (edx.getWord(edlpc + 10) != Edxjava.JWAITKEY)
      throw new EDXError("JWAITKEY with old version of edxjava.edx");
    final int org = edlpc + 14;
    final char endif = edx.getAddr(edlpc + 12);	// address of endif
    if (org + cmds.length * 2 + 14 > endif)
      throw new EDXError("Command sequence too large for JWAITKEY");
    int pc = org;
    for (int i = 0; i < cmds.length; ++i) {
      edx.putWord(pc,cmds[i]);
      pc += 2;
    }
    edx.putBytes(pc,calluser);	pc += calluser.length;
    edx.putWord(pc,edx.getWord(edlpc - 4));	pc += 2;
    edx.putWord(pc,Edxjava.JRESUME);
    if (false && pc + 4 <= endif) {
      pc += 2;
      // help resync when resuming with no suspended request
      edx.putWord(pc,0x00A0);	pc += 2;
      edx.putWord(pc,edlpc);
    }
    calledl(Edxjava.JWAITKEY);
  }

  /** Call back to EDL.  Suspends this EDXServletRequest and transfers
     control back to EDL.  The EDL code returns to Java with an 
     unconditional transfer to RETJAVA.
     @param op	An op code which is put into $TCBCO2 to select
     		what to do. 
   */
  public synchronized void calledx(int op) {
    closeTermSocket();
    edx.ptcbw(Edxtcb.TCBCO2,op);
    calledl(Edxjava.JCALLEDX);
  }

  /** Call an EDL subroutine.  
    @param addr	The EDL address of the SUBROUT to call
   */
  public synchronized void callsub(char addr) {
    final char edlpc = edx.gtcba(Edxtcb.TCBS1);
    final char subrout = edx.getAddr(edlpc - 2);
    final char retadr = edx.getAddr(subrout - 2);	// save return address
    runedl(new char[] { 0x009e, addr });
    edx.putWord(subrout - 2,retadr);		// restore return address
  }

  private void calledl(int op) {
    final char edlpc = edx.gtcba(Edxtcb.TCBS1);
    // push this request on a stack of suspended request by TCB
    next = (EdxjavaRequest)suspended.put(edx,this);
    try {
      m.uidone(op);
      this.m = null;
      while (this.m == null)
	wait();
      /* FIXME: should check edx.getPROG().getBMSOverlayLevel(),
         and throw some kind of exception for skipped levels.
	 Maybe stacks should be by PROG to catch more such errors.
       */
      if (next != null)
        suspended.put(edx,next);
      edx.ptcbw(Edxtcb.TCBS1,edlpc);
    }
    catch (InterruptedException e) {
      throw new EDXError("EDL callback interrupted");
    }
    finally {
      if (next == null)
	suspended.remove(edx);
      else
        suspended.put(edx,next);
    }
  }

  /** Resume the EdxjavaRequest for an ActiveTCB. */
  static void resume(ActiveTCB edx,Edxuser.UIMsg m) {
    /* FIXME: use a list (it should be small), and check whether TCB's 
       are still valid while looking for the EdxjavaRequest to resume.  
       This will handle cancelling the EDL program with a callback active.
     */
    EdxjavaRequest r = (EdxjavaRequest)suspended.remove(edx);
    if (r == null) {	// there is no suspended EdxjavaRequest for this TCB
      m.uidone(Edxjava.JEDXERR);
      return;
    }
    synchronized (r) {
      r.m = m;
      r.notify();
    }
  }

  /** Suspend this BMS overlay program to call another EDX program.
    @param prog	The NAME,VOL of the program to call.
    @param b	a byte[] to send and receive XPARM parameter bytes
    @return the number of XPARM parameter bytes returned
    @exception XParmException on XPARM overflow or not a BMS overlay
    @exception EDXShellException on SHELL stack overflow,
    	or other EDX tasks active in the EDX overlay program
    @exception EDXError on IPC failure talking to the EDX VM
   */
  public synchronized int shell(String prog,byte[] b)
    throws EDXShellException {
    closeTermSocket();
    if (sock != null)
      throw new EDXError("Attempt to SHELL with terminal stream active.");
    PROG prg = edx.getPROG();
    if (!prg.isBMSOverlay())
      throw new XParmException("Not a BMS overlay");
    checkTasks();
    char pplp = edx.getAddr(prg.plp + prg.PRGPPLP);
    // save current EDX Volume and dynamic storage pointer in program header
    edx.putBytes(prg.plp + prg.PRGVOL, edx.getBytes(pplp + prg.PRGVOL,6));
    edx.putDword(prg.plp + prg.PRGSTG,edx.getDword(pplp + prg.PRGSTG));
    // compute program length in pages
    int pgmlen = (edx.getAddr(prg.plp + prg.PRGSTG) - prg.plp + 255) >> 8;
    int dscb = (char)prg.getParm(3);	// disk stack DSCB
    int cm = (char)prg.getParm(5);
    byte[] stksav = edx.getBytes(cm + PROG.CMSTKSAV,36);
    try {
      edx.reset(edx.acttcb + Edxtcb.TCBEEC);
      prg.postend(-1);
    }
    catch (EDXException x) {
      throw new EDXError("IPC failure on SHELL");
    }
    edx.putBytes(dscb + Edxdscb.DSCBORN,stksav);
    int next = edx.getDword(dscb + Edxdscb.DSCBNEX);	// next record
    next += pgmlen + 16;	// reserve room for saved EDX program
    if (next > edx.getDword(dscb + Edxdscb.DSCBLNG))
      throw new EDXShellException("SHELL stack overflow");
    prg.parmPut(b);	// pass parameter info
    edx.putDword(dscb + Edxdscb.DSCBNEX,next);
    edx.putWord(dscb + Edxdscb.DSCBARC,pgmlen);
    // put program name into dscb
    StringTokenizer tok = new StringTokenizer(prog,",");
    edx.atoe(dscb + Edxdscb.DSCBNAM,8,tok.nextToken());
    edx.atoe(dscb + Edxdscb.DSCBVOL,6,tok.hasMoreTokens()?tok.nextToken():"");
    // tell stub to swap out this program and run new program
    termctrl(0x1C30,0x00AA);	// TERMCTRL DISPLAY; WAIT TIMER
    termctrl(0x0016,-1);	// DETACH 
    // control returns here when EDX overlay exits
    edx.ptcbw(Edxtcb.TCBPLP,prg.plp);
    int rc = edx.getWord(cm + PROG.CMCODE);
    //System.err.println("$PARM2="+rc);
    prg.putParm(2,rc);	// $PARM2 = return code
    edx.putBytes(cm + PROG.CMSTKSAV,stksav);
    // restore Volume and Dynamic storage
    edx.putBytes(pplp + prg.PRGVOL,edx.getBytes(prg.plp + prg.PRGVOL,6));
    edx.putDword(pplp + prg.PRGSTG,edx.getDword(prg.plp + prg.PRGSTG));	
    return prg.parmGet(b);
  }

  /** The SHELL call requires that only the main task is active.  Otherwise
      a system program check is likely to occur as swapping out the program
      corrupts the task chain.
   */
  private final void checkTasks() throws EDXShellException {
    // get primary TCB
    int ptcb = edx.getAddr(edx.gtcba(Edxtcb.TCBPLP) + PROG.PRGTCB);
    // check if any other TCBs are active
    for (int tcb = ptcb; (tcb = edx.getAddr(tcb + Edxtcb.TCBPTR)) != ptcb; ) {
      if (edx.getWord(tcb + Edxtcb.TCBEEC) == 0)
        throw new EDXShellException("Secondary tasks active.");
    }
  }

  public int ushell(String cmd,byte[] b) throws EDXShellException {
    try {
      cmd += " #";
      byte[] c = cmd.getBytes("Cp037");
      int clen = c.length;			// command string length
      // some RUNIX versions incorrectly call DTB, shifting parm area
      while (clen > 0 && c[clen - 1] == 0x40) --clen;
      int blen = (b == null) ? 0 : b.length;	// binary parm length
      int len = clen + blen + 2;
      if (len > 256)
	throw new IllegalArgumentException("Command plus parm length > 256");
      // binary parms are passed to RUNIX in unused bytes of a TEXT
      byte[] prm = new byte[len];
      prm[0] = (byte)(len - 2);		// total TEXT length
      prm[1] = (byte)clen;		// bytes used for command string
      System.arraycopy(c,0,prm,2,clen);
      if (blen > 0)
	System.arraycopy(b,0,prm,clen + 2,blen);
      int rc = shell("RUNIX,MSLIB",prm);
      if (b != null)
	System.arraycopy(prm,clen + 2,b,0,blen);
      return rc;
    }
    catch (IOException x) { throw new EDXError("Cp037 encoding not present"); }
  }

  private char edlpc;
  private synchronized Socket getTermSocket() throws IOException {
    if (sock == null) {
      if (edx == null) return null;
      try {
	ServerSocket server = new ServerSocket(0);
	int port = server.getLocalPort();
	edx.ptcbw(Edxtcb.TCBCO2,(short)port);
	edlpc = edx.gtcba(Edxtcb.TCBS1);
        //System.err.println("edlpc = " + Integer.toHexString(edlpc));
	next = (EdxjavaRequest)suspended.put(edx,this);
	m.uidone(Edxjava.JTERM);
	m = null;	// this CALLUSER message done
	//System.err.println("Waiting for connection");
	sock = server.accept();
	//System.err.println("Connected");
	/* hack to simulate GOTO RETJAVA on old edxjava.e
	   Can fail if javaterm immediately terminates after connect.
	 */
	if (edx.getWord(edx.gtcba(Edxtcb.TCBS1)) == 0x009F)
	  edx.ptcbw(Edxtcb.TCBS1,edlpc + 18);
	server.close();
      }
      catch (IOException e) {
	System.err.println("getTermSocket: " + e.getMessage());
	throw e;
      }
    }
    return sock;
  }

  private synchronized void closeTermSocket() {
    if (sock != null) {
      // FIXME: how to trigger ERRPOST in this case?
      try {
	//System.err.println("close term socket");
        sock.close();
        //System.err.println("wait for RUNCMD to return");
	while (m == null)	// wait for RUNCMD to return
	  wait();
        //System.err.println("restore edlpc = " + Integer.toHexString(edlpc));
	edx.ptcbw(Edxtcb.TCBS1,edlpc);
      }
      catch (IOException e) { e.printStackTrace(); }
      catch (InterruptedException x) {
        throw new EDXError("Interrupted waiting for RUNCMD to return");
      }
      finally {
	sock = null;
	if (next == null)
	  suspended.remove(edx);
	else
	  suspended.put(edx,next);
      }
    }
  }

  public OutputStream getOutputStream() throws IOException {
    return getTermSocket().getOutputStream();
  }

  public InputStream getInputStream() throws IOException {
    return getTermSocket().getInputStream();
  }

  public synchronized EDX4978 getScreen() {
    if (screen == null && edx.getCCB().ownedBy(edx) && m != null)
      screen = new EDX4978(edx);
    return screen;
  }

  public final byte[] getParm() {
    return b;
  }

  public final void setParm(byte[] o) {
    if (o.length > outlen)
      outlen = o.length;
    if (outlen > b.length)
      outlen = b.length;
    System.arraycopy(o,0,b,0,outlen);
  }

  public final void setParmLen(int len) {
    outlen = len;
    if (outlen > b.length)
      outlen = b.length;
  }

  /** Release resources used by this EdxjavaRequest.  Close any terminal
    I/O streams created with <code>getOutputStream</code> or
    <code>getInputStream</code>.  Return to request pool.
    */
  private synchronized void dispose(int rc) {
    try {
      closeTermSocket();
      if (m != null) {
	m.uidone(rc);
      }
    }
    finally {
      m = null;
      screen = null;
      s = null;
      edx = null;
      j = null;
      putRequest(this);
    }
  }

  public ActiveTCB getTCB() { return edx; }

  /** Get $PARM1 from the calling program which is the address
    of a BTCB opened to the default BTASDIR in the BMS EDX
    environment.  Ask the EDL JAVA stub program to translate this
    to a pathname.  Return this pathname form of BTASDIR.
    @return null if unavailable (no $PARM1 for example).
   */
  public String getBtasdir() {
    PROG prg = edx.getPROG();
    int prmcnt = prg.getParmCount();
    if (prmcnt < 1 || prmcnt > 5)
      return null;	// only available with PARM=1 through PARM=5
    char btcb = (char)prg.getParm(1);
    if (btcb == 0) return null;	// $PARM1 not set
    byte[] b = edx.getBytes(btcb,32);	// get BTCB bytes
    return j.getBtasdir(b);	// translate through callback
  }

}
