package bmsi.edx;
import bmsi.db.Dataset;
import bmsi.db.IntegerField;
import bmsi.db.StringField;

/** The EDX menu option database.
    @author Stuart D. Gathman
    Copyright (C) 1998 Business Management Systems, Inc.
 */
public class MenuFile extends Dataset {

  /** The menu name. */
  public final StringField scrn = new StringField(this,"scrn",0,8);

  /** The menu option. */
  public final StringField opt = new StringField(this,"opt",8,6);

  /** The EDL program or menu to run when the option is selected.
      If the program name starts with '%', it is taken as the name of 
      a sub-menu.
   */
  public final StringField prog = new StringField(this,"prog",14,8);

  /** The volume or menu subdirectory of the program or menu to run.
      For sub-menus, volumes beginning with '%' are a BTAS sub directory
      containing a <code>$IMAGE</code> menu
      rather than an EDX volume containing a <code>#IMAGE</code> menu.
   */
  public final StringField vol = new StringField(this,"vol",22,6);

  /** The BTAS application base directory to pass to the program when
  it is run.  */
  public final StringField dir = new StringField(this,"dir",28,24);

  /** A description of the menu option.  If the description starts with "!", 
  the text following the "!" is passed as a shell parameter to the
  program when it is run. */
  public final StringField desc = new StringField(this,"desc",52,22);

  /** A 32 bit bit mask of privilege bits that must be set in the
    users privilege mask for this option to be allowed.
    @see MenuUser
   */
  public final IntegerField priv = new IntegerField(this,"priv",74,4);

  public MenuFile() {
    super("/edx/MENU.FILE",true);
  }
}
