package bmsi.edx;
import bmsi.db.Dataset;
import bmsi.db.IntegerField;
import bmsi.db.StringField;

/** The EDX database mapping user ids to EDX user attributes.  Users
  are authenticated by the <code>#MSLOG</code> program. For
  EDX emulators based on Unix, the id of the user running the EDL terminal
  client (edx4978 or edx4974) is used as the EDX user id.
 */

public class MenuUser extends Dataset {
  /** The user id. */
  public IntegerField id = new IntegerField(this,"id",0,2);
  /** The user acct (group). */
  public IntegerField acct = new IntegerField(this,"acct",2,2);
  /** The user privilege mask used to enable certain sensitive
      operations. */
  public IntegerField priv = new IntegerField(this,"priv",4,4);
  /** The users name. */
  public StringField name = new StringField(this,"name",8,24);
  /** The users shell program.  If the name begins with '%', it is assumed to
  be a menu name and <code>#BTMENU</code>
    is used as the shell with the name as an initial menu.
    <p>
    If the shell program is left blank, no shell
    will be loaded and the terminal will have system <code>ATTNLIST</code>
    commands enabled.  The user must then reload <code>#MSMAIN</code> to logout.
   */
  public StringField prog = new StringField(this,"prog",32,8);
  /** The volume to load the shell program from.  For menus (<code>name</code>
  begins with '%'), this is the EDX volume to load the <code>$IMAGE</code>
  menu image from, or the BTAS menu subdirectory to load the <code>#IMAGE</code>
  menu image from.
   */
  public StringField vol = new StringField(this,"vol",40,6);
  /** The users initials - used to tag database changes in some applications.
   */
  public StringField init = new StringField(this,"init",46,3);
  /** The users phone number or other contact information. */
  public StringField phone = new StringField(this,"phone",49,18);
  public MenuUser() {
    super("/edx/MENU.USER",true);
  }
}
