package bmsi.edx.fs;

import bmsi.db.Field;
import bmsi.db.Relate;
import bmsi.db.DataListener;
import bmsi.db.DataScroller;
import bmsi.db.DataBuffer;

/** Maps a Dataset field to an FSPanel screen field.
    @author Stuart D. Gathman
    Copyright (C) 1999 Business Management Systems, Inc.
 */
public class DBField extends FSString implements DataListener {
  private Field fld;
  private boolean modified = false;
  private boolean enabled = false;
  private Dep paintListener;

  public DBField(FSPanel pan,Field fld,int f) {
    super(pan,f);
    this.fld = fld;
    DataBuffer ds = fld.getDataBuffer();
    if (ds instanceof DataScroller)
      ((DataScroller)ds).addDataListener(this);
    else
      enabled = true;
    fld.displayLength(panel.getFieldLen(f));
  }

  public class Dep extends FSString implements PFKeyListener {
    private Relate rel;
    private Field fld;
    private boolean valid;
    public Dep(Field fld,int f) {
      super(DBField.this.panel,f);
      this.fld = fld;
      rel = new Relate(fld.getDataset(),new Field[] { DBField.this.fld },false);
      DBField.this.paintListener = this;
    }
    public void pfkeyPressed(PFKeyEvent e) {
      if (e.pfkey == e.FIELD_MODIFIED)
	paint();
    }
    public void paint() {
      valid = rel.keyChange();
      String s = valid ? fld.asString() : "";
      panel.putField(super.fld,s,true);
    }
    public boolean isValid() { return valid; }
  }

  public void paint() {
    panel.putField(super.fld,fld.asString(),!enabled);
    final Dep l = paintListener;
    if (l != null) l.paint();
  }


  public void recordChanged(DataScroller ds,Field f) {
    //System.err.println("fld = " + fld + ", f = " + f);
    if (fld == null) return;
    if (f == null || f == fld || fld.isCalculated()) {
      paint();
    }
  }

  public void editingChanged(DataScroller ds) {
    enabled = ds.isEditing() && !fld.isReadonly();
    //System.err.println(fld.fieldName() + " enabled: " + enabled);
    paint();
  }

  public void activeChanged(DataScroller ds) { }

  public synchronized void updateData(DataScroller ds) {
    if (modified) {
      try {
	fld.asString(getString());
      }
      catch (NumberFormatException n) {
        posCursor();
      }
      modified = false;
    }
  }

  /** Set the displayed value of the field.  Mark modified so that
     updateData() will copy the new value to the DataBuffer.
   */
  public void setString(String s) {
    super.setString(s);
    modified = true;
  }

  public void datasetScrolled(DataScroller ds,int distance) {
    recordChanged(ds,null);
  }

  public void processPfkey(PFKeyEvent e) {
    if (e.pfkey == e.FIELD_MODIFIED && !fld.isReadonly()) {
      modified = true;
      updateData(null);
    }
    super.processPfkey(e);
  }

}
