package bmsi.edx.fs;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.DateFormat;

/** A self updating clock field.  The clock updates as pfkeys are pressed.
 */
public class FSClock extends FSComp implements PFKeyListener {
  private int fld;
  private final Date ts = new Date();
  private static final DateFormat fmt = 
  	new SimpleDateFormat("MM/dd/yy hh:mm aa");

  public FSClock(FSPanel pan,int fld) {
    super(pan);
    this.fld = fld;
    pan.addPFKeyListener(this);
  }

  /** We don't want to respond to any events specifically for us.
     So always return false and update the clock on any other events. */
  public boolean contains(int row,int col) { return false; }

  public void paint() {
    String s = fmt.format(ts);
    panel.putField(fld,s.substring(0,8),true);
    panel.putField(fld+1,s.substring(9),true);
  }

  public void pfkeyPressed(PFKeyEvent e) {
    long nt = System.currentTimeMillis();
    if (nt - ts.getTime() > 60000) {
      ts.setTime(nt);
      paint();
    }
  }

}
