/*
 * $Log: FSComp.java,v $
 * Revision 1.4  2002/01/29 21:46:45  stuart
 * validate() framework
 *
 * Revision 1.3  2000/11/28 18:00:15  stuart
 * make add/remove listeners atomic again
 *
 * Revision 1.2  2000/11/28 17:55:01  stuart
 * avoid duplicate listeners
 * add isValid()
 *
 * Revision 1.1  2000/11/28 17:53:04  stuart
 * Initial revision
 *
 */
package bmsi.edx.fs;

/** Basic capabilities of screen components for the EDX user interface.
  @author Stuart D. Gathman
  Copyright (C) 2000 Business Management Systems, Inc.
 */

public abstract class FSComp {
  protected FSPanel panel;
  private PFKeyListener[] list;
  private boolean invalid;
  
  protected FSComp(FSPanel pan) {
    panel = pan;
    if (pan != null)
      pan.addComponent(this);
  }

  public synchronized void addPFKeyListener(PFKeyListener l) {
    if (l == null) return;
    if (list != null) for (int i = 0; i < list.length; ++i)
      if (list[i] == l) return;	// already a listener
    int len = (list == null) ? 0 : list.length;
    PFKeyListener[] nl = new PFKeyListener[len + 1];
    if (len > 0)
      System.arraycopy(list,0,nl,0,len);
    nl[len] = l;
    list = nl;
  }

  public synchronized void removePFKeyListener(PFKeyListener l) {
    if (l == null || list == null) return;
    for (int i = 0; i < list.length; ++i) {
      if (list[i] == l) {
        PFKeyListener[] nl = new PFKeyListener[list.length - 1];
	System.arraycopy(list,0,nl,0,i);
	System.arraycopy(list,i+1,nl,i,nl.length - i);
	list = nl;
	break;
      }
    }
  }


  public boolean isValid() { return !invalid; }

  public void setValid(boolean f) { invalid = !f; }

  public boolean validate() { return true; }

  private PFMap[] pfmap;

  public abstract boolean contains(int row,int col);

  public abstract void paint();

  public void setPFMap(PFMap[] pfmap) {
    this.pfmap = pfmap;
  }

  public void processPfkey(PFKeyEvent e) {
    if (e.pfkey == e.FIELD_MODIFIED || invalid && e.pfkey == e.FIELD_EXIT) {
      invalid = !validate();
      if (invalid)
        panel.posCursor(e.row,e.col);
    }
    PFKeyListener[] list = this.list;
    if (list != null) {
      for (int i = 0; i < list.length; ++i) {
        list[i].pfkeyPressed(e);
      }
    }
    PFMap[] pfmap = this.pfmap;
    if (pfmap != null) {
      for (int i = 0; i < pfmap.length; ++i) {
        PFMap m = pfmap[i];
        if (m != null && m.pfkey == e.pfkey) {
	  m.listener.pfkeyPressed(e);
	  return;
	}
      }
    }
  }

}
