package bmsi.edx.fs;

/** Groups fields that form a table or grid in an FSPanel.
   @author Stuart D. Gathman
   Copyright (C) 1999 Business Management Systems, Inc.
 */
public abstract class FSGrid extends FSComp {
  protected int beg, end;
  private int cnt;
  protected int minrow;
  private static final String gray;
  static {
    char[] c = new char[80];
    for (int i = 0; i < c.length; ++i)
      //c[i] = 0x2592;
      c[i] = ' ';
    gray = new String(c);
  }
  protected int rowsel, colsel;

  public FSGrid(FSPanel pan,int beg,int end,int lines) {
    super(pan);
    this.beg = beg;
    this.end = end;
    this.cnt = lines;
    minrow = pan.lindup(beg,end,lines);
  }

  public int getRows() { return cnt; }

  // FIXME: copy portions of Panel buffer to avoid calls to paintRow()

  public void deleteRows(int row,int cnt) {
    //System.err.println("Delete row = " + row + " cnt = " + cnt);
    if (row + cnt < this.cnt) {
      row += cnt;
      setFieldRow(row);
      panel.scroll(beg,end,this.cnt - row,-cnt);
      row = this.cnt - cnt;
    }
    cnt = this.cnt;
    for (int i = row; i < cnt; ++i)
      paintRow(i);
  }

  // FIXME: copy portions of Panel buffer to avoid calls to paintRow()

  public void insertRows(int row,int cnt) {
    try {
      //System.err.println("Insert row = " + row + " cnt = " + cnt);
      if (row < 0) row = 0;
      if (row + cnt < this.cnt) {
	setFieldRow(row);
	panel.scroll(beg,end,this.cnt - row - cnt,cnt);
	cnt = row + cnt;
      }
      else
	cnt = this.cnt;
      for (int i = row; i < cnt; ++i)
	paintRow(i);
    }
    catch (RuntimeException x) {
      System.err.println("row = " + row + " cnt = " + cnt);
      throw x;
    }
  }

  /** Return whether a row,col falls within this grid.  As a side effect,
   call setLine() so that pfkey handlers will reference the proper line
   by default. */
  public boolean contains(int row,int col) {
    if (row < minrow || row >= minrow + cnt)
      return false;
    setLine(row - minrow);
    int fld = panel.getFieldAt(row,col);
    if (fld < beg || end < fld)
      return false;
    colsel = fld - beg;
    return true;
  }

  /** Select the active line for this grid.  The first table
    line is line 0. Fields that are part
    of this FSGrid change their row to the active line.
    @param l	The line relative to the start of the table.
   */
  public void setLine(int l) {
    if (l < 0 || l >= cnt)
      throw new IndexOutOfBoundsException("line="+l);
    setFieldRow(l);
    rowsel = l;
  }

  private void setFieldRow(int l) {
    for (int i = beg; i <= end; ++i)
      panel.setFieldRow(i,minrow + l);
  }

  /** Set the active line and column for this grid. */
  public void setSelection(int row,int col) {
    setLine(row);
    panel.posCursor(beg + col);
    //System.err.println("setSel " + row + ',' + col);
  }

  public abstract String getText(int row,int col);

  public void paintRow(int row) {
    //System.err.println("paintRow " + row);
    setLine(row);
    for (int i = beg; i <= end; ++i) {
      String s = getText(row,i - beg);
      if (s == null)
	panel.putField(i,gray,true);
      else
	panel.putField(i,s,false);
    }
  }

  public void paint() {
    int cnt = getRows();
    for (int i = 0; i < cnt; ++i) {
      paintRow(i);
    }
  }

}
