package bmsi.edx.fs;

import java.util.Vector;

/** A Scrollable memo field.
    @author Stuart D. Gathman
    Copyright (C) 1999 Business Management Systems, Inc.
 */
public class FSTextArea extends FSGrid {
  private final Vector v = new Vector();
  private int firstline;
  private boolean modified;	// modified on screen

  public FSTextArea(FSPanel pan,int fld,int lines) {
    super(pan,fld,fld,lines);
  }

  public String getText(int row,int col) {
    int i = row + firstline;
    if (i >= v.size()) return null;
    return (String)v.elementAt(i);
  }

  public String[] getText() {
    String[] s = new String[v.size()];
    v.copyInto(s);
    modified = false;
    return s;
  }

  public void setText(String[] s) {
    v.removeAllElements();
    if (s != null)
      for (int i = 0; i < s.length; ++i)
	v.addElement(s[i]);
    firstline = 0;
    paint();
    modified = false;
  }

  public void addLine(String s) {
    int row = v.size() - firstline;
    v.addElement(s);
    if (row >= 0 && row < getRows())
      paintRow(row);
  }

  /** Return true if the text has been modified since the
     last getText() or setText().
   */
  public boolean isModified() { return modified; }

  public void processPfkey(PFKeyEvent e) {
    super.processPfkey(e);
    if (e.consumed) return;
    int oldtop = firstline;
    int active = firstline + rowsel;
    switch (e.pfkey) {
    case PFKeyEvent.PF_PREV:
      active -= getRows() - 1;
      break;
    case PFKeyEvent.PF_NEXT:
      active += getRows() - 1;
      break;
    case PFKeyEvent.PF_FIND:
      firstline = active;
      break;
    case PFKeyEvent.PF_INSERT:	// insert line
      if (active <= v.size()) {
        v.insertElementAt("",active);
	insertRows(rowsel,1);
	modified = true;
      }
      return;
    case PFKeyEvent.PF_DELETE:	// delete line
      if (active < v.size()) {
	v.removeElementAt(active);
	deleteRows(rowsel,1);
	modified = true;
      }
      return;
    case PFKeyEvent.FIELD_MODIFIED:
      v.setElementAt(panel.getField(beg),firstline + rowsel);
      modified = true;
      return;
    }
    if (active >= v.size()) active = v.size() - 1;
    if (active < 0) active = 0;
    if (active < firstline)
      firstline = active;
    if (active >= firstline + getRows())
      firstline = active - getRows() + 1;
    if (firstline != oldtop)
      paint();
    setSelection(active - firstline,0);
  }
  
}
