package bmsi.edx.fs;

import bmsi.edx.EDXServlet;
import bmsi.edx.EDXServletRequest;
import bmsi.edx.EDX4978;
import bmsi.db.Dataset;
import java.io.IOException;

/** Implement common features of an EDX record at a time edit screen.
  @author Stuart D. Gathman
  Copyright (C) 2002 Business Management Systems, Inc.
 */
public abstract class RecordEdit implements EDXServlet, PFKeyListener {
  protected final FSPanel pan = new FSPanel();
  private String prompt;
  private int keyfld;
  private Dataset mastf;
  private boolean isActive;

  protected RecordEdit(String prompt,int keyfld) {
    this.prompt = prompt;
    this.keyfld = keyfld;
    pan.addPFKeyListener(this);
  }

  protected void setMaster(Dataset mastf) {
    this.mastf = mastf;
  }

  public FSPanel getPanel() { return pan; }

  /** Override to create a new master record. */
  protected abstract void newRecord() throws IOException;
  /** Override to edit an existing master record.  By default,
   calls Dataset.edit() on master file. */
  protected void editRecord() throws IOException { mastf.edit(); }
  protected boolean validate() throws IOException { return true; }
  /** Override to set the base directory of datasets. */
  public void setBtasdir(String dir) { }
  /** Override to open datasets. */
  public abstract void open() throws IOException;
  /** Override to close datasets. */
  public abstract void close();

  protected void modified(int fld) { }
  protected void inquire(int fld) throws IOException { }

  public void init() { }
  public void destroy() { close(); }

  public void show(EDX4978 sp,boolean isMain) {
    try {
      pan.paint(sp);
      open();
      pan.setFldExit(true);
      if (isMain) {
	isActive = true;
	do {
	  pan.centerLine(23,prompt);
	  mastf.editKey();
	  pan.posCursor(keyfld);
	  pan.fprocess();
	} while (isActive);
      }
      else {
	pan.centerLine(23,prompt);
        editRecord();
	pan.fprocess();
      }
    }
    catch (IOException x) {
      x.printStackTrace();
      pan.centerLine(23,x.toString());
      sp.waitkey();
    }
    finally { close(); }
  }

  public short callJava(EDXServletRequest req) {
    setBtasdir(req.getBtasdir());
    EDX4978 sp = req.getScreen();
    show(sp,true);
    return 0;
  }

  public void pfkeyPressed(PFKeyEvent e) {
    try {
      int fld = pan.getFieldAt(e.row,e.col);
      switch (e.pfkey) {
      case PFKeyEvent.PF_INSERT:	// new contract
        newRecord();
	break;
      case PFKeyEvent.PF_ENTER:
        if (fld == keyfld) break;
	if (validate()) {
	  mastf.write();
	  editRecord();
	}
	break;
      case PFKeyEvent.PF_STOP:
        mastf.cancel();
	isActive = (fld != keyfld);
	break;
      case PFKeyEvent.PF_NEXT:
        mastf.next();
	editRecord();
	break;
      case PFKeyEvent.PF_PREV:
        mastf.prev();
	editRecord();
	break;
      case PFKeyEvent.PF_DIVE:
        inquire(fld);
	break;
      case PFKeyEvent.FIELD_MODIFIED:
        if (fld == keyfld) {
	  if (mastf.gotoKey())
	    editRecord();
	  else {
	    mastf.editKey();
	    pan.posCursor(fld);
	  }
	  e.consume();
	}
	else
	  modified(fld);
      }
    }
    catch (IOException x) {
      x.printStackTrace();
    }
  }
}
